C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      integer function get_anc_data(
     &                               lun_met,
     &                               lun_ozone,
     &                               lun_icec,
     &                               lun_sst,
     &                               lun_nisen,
     &                               lun_nises,
     &                               lun_eco_1km, 
     &                               lun_eco_10min,
     &                               cube,
     &                               buf_anc_size,
     &                               max_pixel,
     &                               rlat,
     &                               rlon,
     &                               cube_pw,
     &                               cube_eco,
     &                               cube_ice,
     &                               cube_snow,
     &                               cube_sfctmp,
     &                               cube_msl,
     &                               cube_ugrd,
     &                               cube_vgrd,
     &                               cube_sst,
     &                               qa_bits
     &                              )
      implicit none
      save

c----------------------------------------------------------------------
C!F77
c
c!Description:
C     Routine for reading and putting MODIS cloud mask ancillary
c     data values into arrays for processing.
C
c!Input parameters:
c cube          Current scan cube number
c lun_eco_1km   1 km Ecosystem file handle number
c lun_eco_10min 10 minute Ecosystem file handle number
c cube          Scan cube number
c buf_anc_size  Line and element granule size
C max_pixel     Maximum number of pixels per scan in granule
c rlat          Array containing scan cube of 1km pixel latitude
c               values
c rlon          Array containing scan cube of 1km pixel longitude
c               values
c
c!Output Parameters:
c
c cube_pw       Array containing scan cube of 1km pixel precipitable
c               water values (g/cm2)
c cube_eco      Array containing scan cube of 1km pixel ecosystem
c               values (Olson World Ecosystem Map)
c cube_ice      Array containing scan cube of 1km pixel sea ice
c               concentration values (fraction)
c cube_snow     Array containing scan cube of 1km pixel snow cover
c               values
c cube_sfctmp   Array containing scan cube of 1km pixel surface
c               temperatures - from model (K)
c cube_msl      Array containing scan cube of 1km pixel mean sea level
c               pressure values from model (hPa)
c cube_ugrd     Array containing scan cube of 1km pixel surface wind
c               u component (m/s)
c cube_vgrd     Array containing scan cube of 1km pixel surface wind
c               v component (m/s)
c cube_sst      Array containing scan cube of 1km pixel surface
c               temperatures - from Reynolds Blended SSTs (K)
c qa_bits       10 byte array containing qa bit results
c
c!Revision History:
c
c!Team-Unique Header:
c This software is developed by the MODIS Science Data Support
c Team for the National Aeronautics and Space Administration,
c Goddard Space Flight Center, under contract NAS5-32373.
c
c!References and Credits:
c See Cloud Mask ATBD-MOD-06.
c       message              Error messaging subroutine
c       get_ancillary_cld    Subroutine to read ancillary data
c
c!END
c----------------------------------------------------------------------

      include 'global.inc'

c --- constants
      real		MSN_LAT
      parameter		(MSN_LAT = 43.083)
 
      real		MSN_LON
      parameter		(MSN_LON = -89.305)

      integer		MSN_ECO_10MIN
      parameter		(MSN_ECO_10MIN = 55)


c --- parameters
      integer		lun_met
      integer		lun_ozone
      integer		lun_icec
      integer		lun_sst
      integer		lun_nisen
      integer		lun_nises
      integer		lun_eco_1km
      integer		lun_eco_10min
      integer 		cube
      integer 		max_pixel
      integer 		buf_anc_size(2)
      integer 		cube_snow(NPIXEL,SCANS_CUBE)
      real 		rlat(NPIXEL,SCANS_CUBE)
      real		rlon(NPIXEL,SCANS_CUBE)
      real		cube_pw(NPIXEL,SCANS_CUBE)
      real		cube_ice(NPIXEL,SCANS_CUBE)
      real		cube_sfctmp(NPIXEL,SCANS_CUBE)
      real		cube_ugrd(NPIXEL,SCANS_CUBE)
      real		cube_vgrd(NPIXEL,SCANS_CUBE)
      real		cube_msl(NPIXEL,SCANS_CUBE)
      real		cube_sst(NPIXEL,SCANS_CUBE)
      byte 		cube_eco(NPIXEL,SCANS_CUBE)
      byte		qa_bits(10)

c --- external functions
      integer 		get_ancillary_cld

c --- internal variables
      byte              map_eco(2)
      byte              madison_eco
      byte              ecotest(1,1)
      integer           lat_indx
      integer           lon_indx
      integer           bytloc
      integer		i
      integer		j
      integer		newlin_eco2
      integer		iocheck
      integer		newele_eco2
      integer		nise
      integer		status
      real 		b_lat
      real 		b_lon
      real 		met_land
      real 		met_prmsl
      real 		met_pwat
      real 		met_ugrd
      real 		met_vgrd
      real 		ozone
      real 		icec
      real		sst
      real		met_landtmp
      real		lat_temp
      real		lon_temp
      real 		met_pres(0:25)
      real 		met_temp(0:25)
      real 		met_mixr(0:25)
      real 		plat(1,1)
      real 		plon(1,1)
      logical 		init

c --- Common statement for debug purposes
      integer           h_output
      integer           debug
      common / bug / debug, h_output

c --- time window for use of nise data (7 days in seconds)
      data 		init/ .TRUE. /


c --- set function status to FAILED
      get_anc_data = -1

c --- initialize output paramaters
      iocheck      = 0
      plat(1,1)    = 0.0
      plon(1,1)    = 0.0
      ecotest(1,1) = 0


      do j = 1, SCANS_CUBE
      do i = 1, NPIXEL
         cube_pw(i,j)     = 0.0
         cube_eco(i,j)    = 0
         cube_ice(i,j)    = 0.0
         cube_snow(i,j)   = 0
         cube_sfctmp(i,j) = 0.0
         cube_ugrd(i,j)   = 0.0
         cube_vgrd(i,j)   = 0.0
         cube_msl(i,j)    = 0.0
         cube_sst(i,j)    = 0.0
      enddo
      enddo

c----------------------------------------------------------------------

c --- Extract a scan cube's worth of data from the geolocation file
      do i = 1, SCANS_CUBE
      do j = 1, max_pixel

         lat_temp = rlat(j,i)
         lon_temp = rlon(j,i)

         status = get_ancillary_cld(
     &                           lun_met,
     &                           lun_ozone,
     &                           lun_icec,
     &                           lun_sst,
     &                           lun_nisen,
     &                           lun_nises,
     &                           lat_temp,
     &                           lon_temp,
     &                           met_pres,
     &                           met_temp,
     &                           met_mixr,
     &                           met_land,
     &                           met_landtmp,
     &                           met_prmsl,
     &                           met_pwat,
     &                           met_ugrd,
     &                           met_vgrd,
     &                           ozone,
     &                           icec,
     &                           sst,
     &                           nise
     &                         )
         if (status.lt.0) then
            call message( 'get_anc_data',
     &                    'Error reading ancillary files.',
     &                    0, 2 )
            return
         endif


c ------ fill the output arrays 
         cube_sfctmp(j,i) = met_landtmp
         cube_ice(j,i)    = icec
         cube_snow(j,i)   = nise
         cube_pw(j,i)     = met_pwat
         cube_ugrd(j,i)   = met_ugrd
         cube_vgrd(j,i)   = met_vgrd
         cube_msl(j,i)    = met_prmsl
         cube_sst(j,i)    = sst

      enddo
      enddo

c----------------------------------------------------------------------

 
c --- set QA bits ...

c --- ...for NCEP data 
      if( met_landtmp.lt.0.0 ) then

c ------ First land surface temp not used
         call set_qa_bit(qa_bits,58)
         call set_qa_bit(qa_bits,59)

c ------ Surface winds - not used
         call set_qa_bit(qa_bits,62)
         call set_qa_bit(qa_bits,63)

c ------ Precipitable water - not used
         call set_qa_bit(qa_bits,73)
         call set_qa_bit(qa_bits,74)

      endif


c --- ... for Ice ancillary data
      if( icec.lt.0.0 ) then

c ------ Preliminary logic for backup, and setting of qa bits
         call set_qa_bit(qa_bits,68)
         call set_qa_bit(qa_bits,69)

      else
c ------ SSMI product
         call set_qa_bit(qa_bits,68)

      endif


c --- ... for Snow mask - if missing value, else set SSMI use bit
      if (nise .lt. 0) then
         call set_qa_bit(qa_bits,66)
         call set_qa_bit(qa_bits,67)
      else
         call set_qa_bit(qa_bits,66)
      endif


c --- ... for Reynolds blended sst data
      if (sst .lt. 0.0) then
         call set_qa_bit(qa_bits,60)
         call set_qa_bit(qa_bits,61)
      endif


c----------------------------------------------------------------------
 

c --- Read the ecosystem information

c --- First, make sure we are reading from the correct file.
c     Extract Madison Wisconsin eco_type from file
      if (init) then

c ------ check for 1km file 
         if( lun_eco_1km.gt.0 ) then

c --------- stage Madison, WI lat/lon as input
            plat(1,1) = MSN_LAT
            plon(1,1) = MSN_LON

c --------- read the 1km eco file
            call read_goode( 
     &                       1,			! 1st dimension of lat/lon arrays
     &                       1,			! 2nd dimension of lat/lon arrays
     &                       plat,		! latitude array (DD.DDD)
     &                       plon,		! longitude array (DD.DDD)
     &                       lun_eco_1km,	! lun of 1km eco file
     &                       ecotest 		! land type values
     &                     )

c ------ if 1km file does not exist, read from 10min file
         else

c --------- stage Madison, WI lat/lon as input
            b_lat = MSN_LAT
            b_lon = MSN_LON

c --------- compute direct access record number for lat/lon
            lat_indx = int((90.0 - b_lat) * 6.0 + 1.0)
            lon_indx = int((b_lon + 180.0) * 6.0 + 1.0)
            if(lat_indx .gt. 1080) lat_indx = 1080
            if(lon_indx .gt. 2160) lon_indx = 2160
            bytloc = ((lat_indx-1) * 2160) + lon_indx
            newlin_eco2 = bytloc / 2 + 1
            newele_eco2 = mod(bytloc, 2)
            if (mod( bytloc,2) .eq. 0) then
              newlin_eco2 = newlin_eco2 - 1
              newele_eco2 = 2
            endif
  
c --------- read value from 10minute file
            READ (
     &             UNIT=lun_eco_10min, 
     &             REC=newlin_eco2, 
     &             IOSTAT=iocheck
     &           ) map_eco

            if( iocheck.ne.0 ) then
               call message( 'get_anc_data',
     &                       'Error reading ecosystem value from file.',
     &                        0, 2 )
               return
            endif


c --------- use computed index to extract the Madison, WI value
            madison_eco = map_eco(newele_eco2)

c --------- Compare value extracted with known correct value
            if( madison_eco.ne.MSN_ECO_10MIN ) then
               call message( 'get_anc_data',
     &         'Extracted incorrect ecosystem value from file.', 
     &         0, 2 )
               return
            endif

         endif
         init = .false.

      endif


c --- Ecosystem has been validated (above). Extract the ecosystem data
c     from the appropriate file (1km or 10min) for the scan cube

c --- Read ecosystem values from 1km dataset
      if (lun_eco_1km.gt.0) then

         call read_goode( 
     &                    buf_anc_size(1), 
     &                    buf_anc_size(2),
     &                    rlat, 
     &                    rlon, 
     &                    lun_eco_1km, 
     &                    cube_eco 
     &                  )

c --- Read ecosystem values from 10min dataset
      else

         do i = 1, SCANS_CUBE
         do j = 1, max_pixel

c --------- stage lon/lon 
            b_lat = rlat(j,i)
            b_lon = rlon(j,i)

c --------- compute direct access record number to read
            lat_indx = int((90.0 - b_lat) * 6.0 + 1.0)
            lon_indx = int((b_lon + 180.0) * 6.0 + 1.0)
            if(lat_indx .gt. 1080) lat_indx = 1080
            if(lon_indx .gt. 2160) lon_indx = 2160
            bytloc = ((lat_indx-1) * 2160) + lon_indx
            newlin_eco2 = bytloc / 2 + 1
            newele_eco2 = mod(bytloc, 2)
            if (mod( bytloc,2) .eq. 0) then
               newlin_eco2 = newlin_eco2 - 1
               newele_eco2 = 2
            endif

c --------- read value from file
            READ (
     &            UNIT=lun_eco_10min, 
     &            REC=newlin_eco2, 
     &            IOSTAT=iocheck
     &           ) map_eco

            if (iocheck .ne. 0) then
               call message( 'get_anc_data',
     &                       'Error reading ecosystem value from file.',
     &                        0, 2 )
               return
            endif

c --------- write indexed value into scan cube ecosystem array
            cube_eco(j,i) = map_eco(newele_eco2)

          enddo
          enddo

      endif


c----------------------------------------------------------------------


c --- debug 
      if (debug .gt. 3) then
        write(h_output,'(15x,'' ANCILLARY DATA FOR CUBE: '',I5)') cube
        write(h_output,'(2x,'' Ice concentraion'',/,20(5f10.4/))')
     &       ((cube_ice(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Snow from NISE'',/,20(5I10/))')
     &       ((cube_snow(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,''Precipitable Water (g/cm**2)'',
     &       /,20(5f10.4/))') ((cube_pw(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Ecosystem Type '',
     &       /,20(5I10/))') ((cube_eco(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Surface Temperature (K)'',
     &       /,20(5f10.4/))') ((cube_sfctmp(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' U Wind Component (m/s) '',
     &       /,20(5f10.4/))') ((cube_ugrd(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' V Wind Component (m/s) '',
     &       /,20(5f10.4/))') ((cube_vgrd(j,i),j=1,10),i=1,10)
        write(h_output,'(2x,'' Mean Sea Level Pressure (hPa) '',
     &       /,20(5f10.4/))') ((cube_msl(j,i),j=1,10),i=1,10)
      endif


c----------------------------------------------------------------------


c --- set function status to SUCCESS
      get_anc_data = 0

      return
      end
